#ifndef APROTOC_672e13108aa3d23322d5b39cafcd6f3f0a10c9a0ac400a70877c3e30cf05dbdb
#define APROTOC_672e13108aa3d23322d5b39cafcd6f3f0a10c9a0ac400a70877c3e30cf05dbdb

#include <cstdint>
#include <map>
#include <memory>
#include <string>
#include <string_view>
#include <vector>

#include <argdata.h>
#include <arpc++/arpc++.h>

namespace flower {
namespace cat {

class Configuration final : public arpc::Message {
 public:
  Configuration() : buffer_size_(0), listen_(false) {}

  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (buffer_size_ != 0) {
      keys.push_back(argdata_builder->BuildStr("buffer_size"));
      values.push_back(argdata_builder->BuildInt(buffer_size_));
    }
    if (listen_ != false) {
      keys.push_back(argdata_builder->BuildStr("listen"));
      values.push_back(&argdata_true);
    }
    if (logger_output_) {
      keys.push_back(argdata_builder->BuildStr("logger_output"));
      values.push_back(argdata_builder->BuildFd(logger_output_));
    }
    if (stream_input_) {
      keys.push_back(argdata_builder->BuildStr("stream_input"));
      values.push_back(argdata_builder->BuildFd(stream_input_));
    }
    if (stream_output_) {
      keys.push_back(argdata_builder->BuildStr("stream_output"));
      values.push_back(argdata_builder->BuildFd(stream_output_));
    }
    if (switchboard_socket_) {
      keys.push_back(argdata_builder->BuildStr("switchboard_socket"));
      values.push_back(argdata_builder->BuildFd(switchboard_socket_));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = Configuration();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "buffer_size") {
          argdata_get_int(value, &buffer_size_);
        } else if (keyss == "listen") {
          argdata_get_bool(value, &listen_);
        } else if (keyss == "logger_output") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            logger_output_ = std::move(fd);
        } else if (keyss == "stream_input") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            stream_input_ = std::move(fd);
        } else if (keyss == "stream_output") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            stream_output_ = std::move(fd);
        } else if (keyss == "switchboard_socket") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            switchboard_socket_ = std::move(fd);
        }
      }
      argdata_map_next(&it);
    }
  }

  std::uint32_t buffer_size() const { return buffer_size_; }
  void set_buffer_size(std::uint32_t value) { buffer_size_ = value; }
  void clear_buffer_size() { buffer_size_ = 0; }

  bool listen() const { return listen_; }
  void set_listen(bool value) { listen_ = value; }
  void clear_listen() { listen_ = false; }

  const std::shared_ptr<arpc::FileDescriptor>& logger_output() const { return logger_output_; }
  void set_logger_output(const std::shared_ptr<arpc::FileDescriptor>& value) { logger_output_ = value; }
  void clear_logger_output() { logger_output_.reset(); }

  const std::shared_ptr<arpc::FileDescriptor>& stream_input() const { return stream_input_; }
  void set_stream_input(const std::shared_ptr<arpc::FileDescriptor>& value) { stream_input_ = value; }
  void clear_stream_input() { stream_input_.reset(); }

  const std::shared_ptr<arpc::FileDescriptor>& stream_output() const { return stream_output_; }
  void set_stream_output(const std::shared_ptr<arpc::FileDescriptor>& value) { stream_output_ = value; }
  void clear_stream_output() { stream_output_.reset(); }

  const std::shared_ptr<arpc::FileDescriptor>& switchboard_socket() const { return switchboard_socket_; }
  void set_switchboard_socket(const std::shared_ptr<arpc::FileDescriptor>& value) { switchboard_socket_ = value; }
  void clear_switchboard_socket() { switchboard_socket_.reset(); }

 private:
  std::uint32_t buffer_size_;
  bool listen_;
  std::shared_ptr<arpc::FileDescriptor> logger_output_;
  std::shared_ptr<arpc::FileDescriptor> stream_input_;
  std::shared_ptr<arpc::FileDescriptor> stream_output_;
  std::shared_ptr<arpc::FileDescriptor> switchboard_socket_;
};

}  // namespace cat
}  // namespace flower

#endif
