#ifndef APROTOC_eb4dc9dcb9985ee4a9a320fe1c822107bd468362a97431d540a2472981396936
#define APROTOC_eb4dc9dcb9985ee4a9a320fe1c822107bd468362a97431d540a2472981396936

#include <cstdint>
#include <map>
#include <memory>
#include <string>
#include <string_view>
#include <vector>

#include <argdata.h>
#include <arpc++/arpc++.h>

namespace flower {
namespace ls {

class Configuration final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (list_output_) {
      keys.push_back(argdata_builder->BuildStr("list_output"));
      values.push_back(argdata_builder->BuildFd(list_output_));
    }
    if (logger_output_) {
      keys.push_back(argdata_builder->BuildStr("logger_output"));
      values.push_back(argdata_builder->BuildFd(logger_output_));
    }
    if (switchboard_socket_) {
      keys.push_back(argdata_builder->BuildStr("switchboard_socket"));
      values.push_back(argdata_builder->BuildFd(switchboard_socket_));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = Configuration();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "list_output") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            list_output_ = std::move(fd);
        } else if (keyss == "logger_output") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            logger_output_ = std::move(fd);
        } else if (keyss == "switchboard_socket") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            switchboard_socket_ = std::move(fd);
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::shared_ptr<arpc::FileDescriptor>& list_output() const { return list_output_; }
  void set_list_output(const std::shared_ptr<arpc::FileDescriptor>& value) { list_output_ = value; }
  void clear_list_output() { list_output_.reset(); }

  const std::shared_ptr<arpc::FileDescriptor>& logger_output() const { return logger_output_; }
  void set_logger_output(const std::shared_ptr<arpc::FileDescriptor>& value) { logger_output_ = value; }
  void clear_logger_output() { logger_output_.reset(); }

  const std::shared_ptr<arpc::FileDescriptor>& switchboard_socket() const { return switchboard_socket_; }
  void set_switchboard_socket(const std::shared_ptr<arpc::FileDescriptor>& value) { switchboard_socket_ = value; }
  void clear_switchboard_socket() { switchboard_socket_.reset(); }

 private:
  std::shared_ptr<arpc::FileDescriptor> list_output_;
  std::shared_ptr<arpc::FileDescriptor> logger_output_;
  std::shared_ptr<arpc::FileDescriptor> switchboard_socket_;
};

}  // namespace ls
}  // namespace flower

#endif
