#ifndef APROTOC_26718f38a036e523d4637c2959a2abfa418bbb0fcfa272e13e5262509923edb6
#define APROTOC_26718f38a036e523d4637c2959a2abfa418bbb0fcfa272e13e5262509923edb6

#include <cstdint>
#include <map>
#include <memory>
#include <string>
#include <string_view>
#include <vector>

#include <argdata.h>
#include <arpc++/arpc++.h>

namespace flower {
namespace protocol {
namespace resolver {

class ResolveRequest final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (!resolve_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("resolve_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : resolve_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ResolveRequest();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "resolve_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                resolve_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::map<std::string, std::string, std::less<>>& resolve_labels() const { return resolve_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_resolve_labels() { return &resolve_labels_; }

 private:
  std::map<std::string, std::string, std::less<>> resolve_labels_;
};

class Target final : public arpc::Message {
 public:
  Target() : weight_(0) {}

  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (!out_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("out_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : out_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    if (weight_ != 0) {
      keys.push_back(argdata_builder->BuildStr("weight"));
      values.push_back(argdata_builder->BuildInt(weight_));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = Target();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "out_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                out_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        } else if (keyss == "weight") {
          argdata_get_int(value, &weight_);
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::map<std::string, std::string, std::less<>>& out_labels() const { return out_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_out_labels() { return &out_labels_; }

  std::uint32_t weight() const { return weight_; }
  void set_weight(std::uint32_t value) { weight_ = value; }
  void clear_weight() { weight_ = 0; }

 private:
  std::map<std::string, std::string, std::less<>> out_labels_;
  std::uint32_t weight_;
};

class ResolveResponse final : public arpc::Message {
 public:
  ResolveResponse() : max_cache_duration_seconds_(0) {}

  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (max_cache_duration_seconds_ != 0) {
      keys.push_back(argdata_builder->BuildStr("max_cache_duration_seconds"));
      values.push_back(argdata_builder->BuildInt(max_cache_duration_seconds_));
    }
    if (!targets_.empty()) {
      keys.push_back(argdata_builder->BuildStr("targets"));
      std::vector<const argdata_t*> elements;
      for (const auto& element : targets_) {
        elements.push_back(element.Build(argdata_builder));
      }
      values.push_back(argdata_builder->BuildSeq(std::move(elements)));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ResolveResponse();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "max_cache_duration_seconds") {
          argdata_get_int(value, &max_cache_duration_seconds_);
        } else if (keyss == "targets") {
          argdata_seq_iterator_t it2;
          argdata_seq_iterate(value, &it2);
          const argdata_t* element;
          while (argdata_seq_get(&it2, &element)) {
            targets_.emplace_back().Parse(*element, argdata_parser);
            argdata_seq_next(&it2);
          }
        }
      }
      argdata_map_next(&it);
    }
  }

  std::uint32_t max_cache_duration_seconds() const { return max_cache_duration_seconds_; }
  void set_max_cache_duration_seconds(std::uint32_t value) { max_cache_duration_seconds_ = value; }
  void clear_max_cache_duration_seconds() { max_cache_duration_seconds_ = 0; }

  std::size_t targets_size() const { return targets_.size(); }
  const Target& targets(std::size_t index) const { return targets_[index]; }
  Target* mutable_targets(std::size_t index) { return &targets_[index]; }
  Target* add_targets() { return &targets_.emplace_back(); }
  void clear_targets() { targets_.clear(); }
  const std::vector<Target>& targets() const { return targets_; }
  std::vector<Target>* mutable_targets() { return &targets_; }

 private:
  std::uint32_t max_cache_duration_seconds_;
  std::vector<Target> targets_;
};

struct Resolver {

class Service : public arpc::Service {
 public:
  std::string_view GetName() override {
    return "Resolver";
  }

  arpc::Status BlockingUnaryCall(std::string_view rpc, arpc::ServerContext* context, const argdata_t& request, arpc::ArgdataParser* argdata_parser, const argdata_t** response, arpc::ArgdataBuilder* argdata_builder) override {
    if (rpc == "Resolve") {
      ResolveRequest request_object;
      request_object.Parse(request, argdata_parser);
      ResolveResponse response_object;
      arpc::Status status = Resolve(context, &request_object, &response_object);
      if (status.ok())
        *response = response_object.Build(argdata_builder);
      return status;
    }
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this service");
  }

  arpc::Status BlockingClientStreamingCall(std::string_view rpc, arpc::ServerContext* context, arpc::ServerReaderImpl* reader, const argdata_t** response, arpc::ArgdataBuilder* argdata_builder) override {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this service");
  }

  arpc::Status BlockingServerStreamingCall(std::string_view rpc, arpc::ServerContext* context, const argdata_t& request, arpc::ArgdataParser* argdata_parser, arpc::ServerWriterImpl* writer) override {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this service");
  }

  virtual arpc::Status Resolve(arpc::ServerContext* context, const ResolveRequest* request, ResolveResponse* response) {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this implementation");
  }
};

class Stub {
 public:
  explicit Stub(const std::shared_ptr<arpc::Channel>& channel)
      : channel_(channel) {}

  arpc::Status Resolve(arpc::ClientContext* context, const ResolveRequest& request, ResolveResponse* response) {
    return channel_->BlockingUnaryCall(arpc::RpcMethod("Resolver", "Resolve"), context, request, response);
  }

 private:
  const std::shared_ptr<arpc::Channel> channel_;
};

static std::unique_ptr<Stub> NewStub(const std::shared_ptr<arpc::Channel>& channel) {
  return std::make_unique<Stub>(channel);
}

};

}  // namespace resolver
}  // namespace protocol
}  // namespace flower

#endif
