#ifndef APROTOC_ce3680483aa1b314a9d565c02b7f372c0e6930b9cf66c737d3563fe95d750782
#define APROTOC_ce3680483aa1b314a9d565c02b7f372c0e6930b9cf66c737d3563fe95d750782

#include <cstdint>
#include <map>
#include <memory>
#include <string>
#include <string_view>
#include <vector>

#include <argdata.h>
#include <arpc++/arpc++.h>

namespace flower {
namespace protocol {
namespace server {

class ConnectRequest final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (client_) {
      keys.push_back(argdata_builder->BuildStr("client"));
      values.push_back(argdata_builder->BuildFd(client_));
    }
    if (!connection_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("connection_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : connection_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ConnectRequest();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "client") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            client_ = std::move(fd);
        } else if (keyss == "connection_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                connection_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::shared_ptr<arpc::FileDescriptor>& client() const { return client_; }
  void set_client(const std::shared_ptr<arpc::FileDescriptor>& value) { client_ = value; }
  void clear_client() { client_.reset(); }

  const std::map<std::string, std::string, std::less<>>& connection_labels() const { return connection_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_connection_labels() { return &connection_labels_; }

 private:
  std::shared_ptr<arpc::FileDescriptor> client_;
  std::map<std::string, std::string, std::less<>> connection_labels_;
};

class ConnectResponse final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    return &argdata_null;
  }

  void Clear() override {
    *this = ConnectResponse();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
  }

 private:
};

struct Server {

class Service : public arpc::Service {
 public:
  std::string_view GetName() override {
    return "Server";
  }

  arpc::Status BlockingUnaryCall(std::string_view rpc, arpc::ServerContext* context, const argdata_t& request, arpc::ArgdataParser* argdata_parser, const argdata_t** response, arpc::ArgdataBuilder* argdata_builder) override {
    if (rpc == "Connect") {
      ConnectRequest request_object;
      request_object.Parse(request, argdata_parser);
      ConnectResponse response_object;
      arpc::Status status = Connect(context, &request_object, &response_object);
      if (status.ok())
        *response = response_object.Build(argdata_builder);
      return status;
    }
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this service");
  }

  arpc::Status BlockingClientStreamingCall(std::string_view rpc, arpc::ServerContext* context, arpc::ServerReaderImpl* reader, const argdata_t** response, arpc::ArgdataBuilder* argdata_builder) override {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this service");
  }

  arpc::Status BlockingServerStreamingCall(std::string_view rpc, arpc::ServerContext* context, const argdata_t& request, arpc::ArgdataParser* argdata_parser, arpc::ServerWriterImpl* writer) override {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this service");
  }

  virtual arpc::Status Connect(arpc::ServerContext* context, const ConnectRequest* request, ConnectResponse* response) {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this implementation");
  }
};

class Stub {
 public:
  explicit Stub(const std::shared_ptr<arpc::Channel>& channel)
      : channel_(channel) {}

  arpc::Status Connect(arpc::ClientContext* context, const ConnectRequest& request, ConnectResponse* response) {
    return channel_->BlockingUnaryCall(arpc::RpcMethod("Server", "Connect"), context, request, response);
  }

 private:
  const std::shared_ptr<arpc::Channel> channel_;
};

static std::unique_ptr<Stub> NewStub(const std::shared_ptr<arpc::Channel>& channel) {
  return std::make_unique<Stub>(channel);
}

};

}  // namespace server
}  // namespace protocol
}  // namespace flower

#endif
