#ifndef APROTOC_66c7d684f71543c8b986cf69d4773dc62583c01b987075a9dde429857bea25ca
#define APROTOC_66c7d684f71543c8b986cf69d4773dc62583c01b987075a9dde429857bea25ca

#include <cstdint>
#include <map>
#include <memory>
#include <string>
#include <string_view>
#include <vector>

#include <argdata.h>
#include <arpc++/arpc++.h>

namespace flower {
namespace protocol {
namespace switchboard {

class ClientConnectRequest final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (!out_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("out_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : out_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ClientConnectRequest();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "out_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                out_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::map<std::string, std::string, std::less<>>& out_labels() const { return out_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_out_labels() { return &out_labels_; }

 private:
  std::map<std::string, std::string, std::less<>> out_labels_;
};

class ClientConnectResponse final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (!connection_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("connection_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : connection_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    if (server_) {
      keys.push_back(argdata_builder->BuildStr("server"));
      values.push_back(argdata_builder->BuildFd(server_));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ClientConnectResponse();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "connection_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                connection_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        } else if (keyss == "server") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            server_ = std::move(fd);
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::map<std::string, std::string, std::less<>>& connection_labels() const { return connection_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_connection_labels() { return &connection_labels_; }

  const std::shared_ptr<arpc::FileDescriptor>& server() const { return server_; }
  void set_server(const std::shared_ptr<arpc::FileDescriptor>& value) { server_ = value; }
  void clear_server() { server_.reset(); }

 private:
  std::map<std::string, std::string, std::less<>> connection_labels_;
  std::shared_ptr<arpc::FileDescriptor> server_;
};

class ConstrainResponse final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (switchboard_) {
      keys.push_back(argdata_builder->BuildStr("switchboard"));
      values.push_back(argdata_builder->BuildFd(switchboard_));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ConstrainResponse();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "switchboard") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            switchboard_ = std::move(fd);
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::shared_ptr<arpc::FileDescriptor>& switchboard() const { return switchboard_; }
  void set_switchboard(const std::shared_ptr<arpc::FileDescriptor>& value) { switchboard_ = value; }
  void clear_switchboard() { switchboard_.reset(); }

 private:
  std::shared_ptr<arpc::FileDescriptor> switchboard_;
};

class EgressStartRequest final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (!in_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("in_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : in_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = EgressStartRequest();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "in_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                in_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::map<std::string, std::string, std::less<>>& in_labels() const { return in_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_in_labels() { return &in_labels_; }

 private:
  std::map<std::string, std::string, std::less<>> in_labels_;
};

class EgressStartResponse final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (egress_) {
      keys.push_back(argdata_builder->BuildStr("egress"));
      values.push_back(argdata_builder->BuildFd(egress_));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = EgressStartResponse();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "egress") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            egress_ = std::move(fd);
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::shared_ptr<arpc::FileDescriptor>& egress() const { return egress_; }
  void set_egress(const std::shared_ptr<arpc::FileDescriptor>& value) { egress_ = value; }
  void clear_egress() { egress_.reset(); }

 private:
  std::shared_ptr<arpc::FileDescriptor> egress_;
};

class IngressConnectRequest final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (client_) {
      keys.push_back(argdata_builder->BuildStr("client"));
      values.push_back(argdata_builder->BuildFd(client_));
    }
    if (!out_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("out_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : out_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = IngressConnectRequest();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "client") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            client_ = std::move(fd);
        } else if (keyss == "out_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                out_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::shared_ptr<arpc::FileDescriptor>& client() const { return client_; }
  void set_client(const std::shared_ptr<arpc::FileDescriptor>& value) { client_ = value; }
  void clear_client() { client_.reset(); }

  const std::map<std::string, std::string, std::less<>>& out_labels() const { return out_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_out_labels() { return &out_labels_; }

 private:
  std::shared_ptr<arpc::FileDescriptor> client_;
  std::map<std::string, std::string, std::less<>> out_labels_;
};

class IngressConnectResponse final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (!connection_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("connection_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : connection_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = IngressConnectResponse();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "connection_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                connection_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::map<std::string, std::string, std::less<>>& connection_labels() const { return connection_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_connection_labels() { return &connection_labels_; }

 private:
  std::map<std::string, std::string, std::less<>> connection_labels_;
};

class ListRequest final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (!out_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("out_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : out_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ListRequest();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "out_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                out_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::map<std::string, std::string, std::less<>>& out_labels() const { return out_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_out_labels() { return &out_labels_; }

 private:
  std::map<std::string, std::string, std::less<>> out_labels_;
};

class ListResponse final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (!labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ListResponse();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::map<std::string, std::string, std::less<>>& labels() const { return labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_labels() { return &labels_; }

 private:
  std::map<std::string, std::string, std::less<>> labels_;
};

class ResolverStartRequest final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (!in_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("in_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : in_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    if (!out_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("out_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : out_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    if (!resolve_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("resolve_labels"));
      std::vector<const argdata_t*> elements;
      for (const auto& element : resolve_labels_) {
        elements.push_back(argdata_builder->BuildStr(element));
      }
      values.push_back(argdata_builder->BuildSeq(std::move(elements)));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ResolverStartRequest();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "in_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                in_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        } else if (keyss == "out_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                out_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        } else if (keyss == "resolve_labels") {
          argdata_seq_iterator_t it2;
          argdata_seq_iterate(value, &it2);
          const argdata_t* element;
          while (argdata_seq_get(&it2, &element)) {
            const char* elementstr;
            std::size_t elementlen;
            if (argdata_get_str(element, &elementstr, &elementlen) == 0)
              resolve_labels_.emplace_back(std::string_view(elementstr, elementlen));
            argdata_seq_next(&it2);
          }
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::map<std::string, std::string, std::less<>>& in_labels() const { return in_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_in_labels() { return &in_labels_; }

  const std::map<std::string, std::string, std::less<>>& out_labels() const { return out_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_out_labels() { return &out_labels_; }

  std::size_t resolve_labels_size() const { return resolve_labels_.size(); }
  const std::string& resolve_labels(std::size_t index) const { return resolve_labels_[index]; }
  void set_resolve_labels(std::size_t index, std::string_view value) { resolve_labels_[index] = value; }
  std::string* mutable_resolve_labels(std::size_t index) { return &resolve_labels_[index]; }
  void add_resolve_labels(std::string_view value) { resolve_labels_.emplace_back(value); }
  std::string* add_resolve_labels() { return &resolve_labels_.emplace_back(); }
  void clear_resolve_labels() { resolve_labels_.clear(); }
  const std::vector<std::string>& resolve_labels() const { return resolve_labels_; }
  std::vector<std::string>* mutable_resolve_labels() { return &resolve_labels_; }

 private:
  std::map<std::string, std::string, std::less<>> in_labels_;
  std::map<std::string, std::string, std::less<>> out_labels_;
  std::vector<std::string> resolve_labels_;
};

class ResolverStartResponse final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (resolver_) {
      keys.push_back(argdata_builder->BuildStr("resolver"));
      values.push_back(argdata_builder->BuildFd(resolver_));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ResolverStartResponse();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "resolver") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            resolver_ = std::move(fd);
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::shared_ptr<arpc::FileDescriptor>& resolver() const { return resolver_; }
  void set_resolver(const std::shared_ptr<arpc::FileDescriptor>& value) { resolver_ = value; }
  void clear_resolver() { resolver_.reset(); }

 private:
  std::shared_ptr<arpc::FileDescriptor> resolver_;
};

enum Right {
  CLIENT_CONNECT = 0,
  EGRESS_START = 1,
  INGRESS_CONNECT = 2,
  LIST = 5,
  RESOLVER_START = 3,
  SERVER_START = 4
};

namespace {

inline bool Right_IsValid(int value) {
  return value == 0 || value == 1 || value == 2 || value == 3 || value == 4 || value == 5;
}

inline const char* Right_Name(int value) {
  switch (value) {
  case 0: return "CLIENT_CONNECT";
  case 1: return "EGRESS_START";
  case 2: return "INGRESS_CONNECT";
  case 3: return "RESOLVER_START";
  case 4: return "SERVER_START";
  case 5: return "LIST";
  default: return "";
  }
}

inline bool Right_Parse(std::string_view name, Right* value) {
  if (name == "CLIENT_CONNECT") { *value = Right::CLIENT_CONNECT; return true; }
  if (name == "EGRESS_START") { *value = Right::EGRESS_START; return true; }
  if (name == "INGRESS_CONNECT") { *value = Right::INGRESS_CONNECT; return true; }
  if (name == "LIST") { *value = Right::LIST; return true; }
  if (name == "RESOLVER_START") { *value = Right::RESOLVER_START; return true; }
  if (name == "SERVER_START") { *value = Right::SERVER_START; return true; }
  return false;
}

const Right Right_MIN = Right::CLIENT_CONNECT;
const Right Right_MAX = Right::LIST;
const std::size_t Right_ARRAYSIZE = 6;

}  // namespace

class ServerStartRequest final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (!in_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("in_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : in_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ServerStartRequest();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "in_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                in_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::map<std::string, std::string, std::less<>>& in_labels() const { return in_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_in_labels() { return &in_labels_; }

 private:
  std::map<std::string, std::string, std::less<>> in_labels_;
};

class ServerStartResponse final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (server_) {
      keys.push_back(argdata_builder->BuildStr("server"));
      values.push_back(argdata_builder->BuildFd(server_));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ServerStartResponse();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "server") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            server_ = std::move(fd);
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::shared_ptr<arpc::FileDescriptor>& server() const { return server_; }
  void set_server(const std::shared_ptr<arpc::FileDescriptor>& value) { server_ = value; }
  void clear_server() { server_.reset(); }

 private:
  std::shared_ptr<arpc::FileDescriptor> server_;
};

class ConstrainRequest final : public arpc::Message {
 public:
  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (!in_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("in_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : in_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    if (!out_labels_.empty()) {
      keys.push_back(argdata_builder->BuildStr("out_labels"));
      std::vector<const argdata_t*> mapkeys;
      std::vector<const argdata_t*> mapvalues;
      for (const auto& mapentry : out_labels_) {
        mapkeys.push_back(argdata_builder->BuildStr(mapentry.first));
        mapvalues.push_back(argdata_builder->BuildStr(mapentry.second));
      }
      values.push_back(argdata_builder->BuildMap(std::move(mapkeys), std::move(mapvalues)));
    }
    if (!rights_.empty()) {
      keys.push_back(argdata_builder->BuildStr("rights"));
      std::vector<const argdata_t*> elements;
      for (const auto& element : rights_) {
        elements.push_back(argdata_builder->BuildStr(Right_Name(element)));
      }
      values.push_back(argdata_builder->BuildSeq(std::move(elements)));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = ConstrainRequest();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "in_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                in_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        } else if (keyss == "out_labels") {
          argdata_map_iterator_t it2;
          argdata_map_iterate(value, &it2);
          const argdata_t* key2, *value2;
          while (argdata_map_get(&it2, &key2, &value2)) {
            const char* key2str;
            std::size_t key2len;
            if (argdata_get_str(key2, &key2str, &key2len) == 0) {
              std::string_view mapkey(key2str, key2len);
              const char* value2str;
              std::size_t value2len;
              if (argdata_get_str(value2, &value2str, &value2len) == 0)
                out_labels_.emplace(mapkey, std::string()).first->second = std::string_view(value2str, value2len);
            }
            argdata_map_next(&it2);
          }
        } else if (keyss == "rights") {
          argdata_seq_iterator_t it2;
          argdata_seq_iterate(value, &it2);
          const argdata_t* element;
          while (argdata_seq_get(&it2, &element)) {
            const char* elementstr;
            std::size_t elementlen;
            if (argdata_get_str(element, &elementstr, &elementlen) == 0)
              Right_Parse(std::string_view(elementstr, elementlen), &rights_.emplace_back(Right::CLIENT_CONNECT));
            argdata_seq_next(&it2);
          }
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::map<std::string, std::string, std::less<>>& in_labels() const { return in_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_in_labels() { return &in_labels_; }

  const std::map<std::string, std::string, std::less<>>& out_labels() const { return out_labels_; }
  std::map<std::string, std::string, std::less<>>* mutable_out_labels() { return &out_labels_; }

  std::size_t rights_size() const { return rights_.size(); }
  Right rights(std::size_t index) const { return rights_[index]; }
  void set_rights(std::size_t index, Right value) { rights_[index] = value; }
  void add_rights(Right value) { return rights_.push_back(value); }
  void clear_rights() { rights_.clear(); }
  const std::vector<Right>& rights() const { return rights_; }
  std::vector<Right>* mutable_rights() { return &rights_; }

 private:
  std::map<std::string, std::string, std::less<>> in_labels_;
  std::map<std::string, std::string, std::less<>> out_labels_;
  std::vector<Right> rights_;
};

struct Switchboard {

class Service : public arpc::Service {
 public:
  std::string_view GetName() override {
    return "Switchboard";
  }

  arpc::Status BlockingUnaryCall(std::string_view rpc, arpc::ServerContext* context, const argdata_t& request, arpc::ArgdataParser* argdata_parser, const argdata_t** response, arpc::ArgdataBuilder* argdata_builder) override {
    if (rpc == "ClientConnect") {
      ClientConnectRequest request_object;
      request_object.Parse(request, argdata_parser);
      ClientConnectResponse response_object;
      arpc::Status status = ClientConnect(context, &request_object, &response_object);
      if (status.ok())
        *response = response_object.Build(argdata_builder);
      return status;
    }
    if (rpc == "Constrain") {
      ConstrainRequest request_object;
      request_object.Parse(request, argdata_parser);
      ConstrainResponse response_object;
      arpc::Status status = Constrain(context, &request_object, &response_object);
      if (status.ok())
        *response = response_object.Build(argdata_builder);
      return status;
    }
    if (rpc == "EgressStart") {
      EgressStartRequest request_object;
      request_object.Parse(request, argdata_parser);
      EgressStartResponse response_object;
      arpc::Status status = EgressStart(context, &request_object, &response_object);
      if (status.ok())
        *response = response_object.Build(argdata_builder);
      return status;
    }
    if (rpc == "IngressConnect") {
      IngressConnectRequest request_object;
      request_object.Parse(request, argdata_parser);
      IngressConnectResponse response_object;
      arpc::Status status = IngressConnect(context, &request_object, &response_object);
      if (status.ok())
        *response = response_object.Build(argdata_builder);
      return status;
    }
    if (rpc == "ResolverStart") {
      ResolverStartRequest request_object;
      request_object.Parse(request, argdata_parser);
      ResolverStartResponse response_object;
      arpc::Status status = ResolverStart(context, &request_object, &response_object);
      if (status.ok())
        *response = response_object.Build(argdata_builder);
      return status;
    }
    if (rpc == "ServerStart") {
      ServerStartRequest request_object;
      request_object.Parse(request, argdata_parser);
      ServerStartResponse response_object;
      arpc::Status status = ServerStart(context, &request_object, &response_object);
      if (status.ok())
        *response = response_object.Build(argdata_builder);
      return status;
    }
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this service");
  }

  arpc::Status BlockingClientStreamingCall(std::string_view rpc, arpc::ServerContext* context, arpc::ServerReaderImpl* reader, const argdata_t** response, arpc::ArgdataBuilder* argdata_builder) override {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this service");
  }

  arpc::Status BlockingServerStreamingCall(std::string_view rpc, arpc::ServerContext* context, const argdata_t& request, arpc::ArgdataParser* argdata_parser, arpc::ServerWriterImpl* writer) override {
    if (rpc == "List") {
      ListRequest request_object;
      request_object.Parse(request, argdata_parser);
      arpc::ServerWriter<ListResponse> writer_object(writer);
      return List(context, &request_object, &writer_object);
    }
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this service");
  }

  virtual arpc::Status ClientConnect(arpc::ServerContext* context, const ClientConnectRequest* request, ClientConnectResponse* response) {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this implementation");
  }

  virtual arpc::Status Constrain(arpc::ServerContext* context, const ConstrainRequest* request, ConstrainResponse* response) {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this implementation");
  }

  virtual arpc::Status EgressStart(arpc::ServerContext* context, const EgressStartRequest* request, EgressStartResponse* response) {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this implementation");
  }

  virtual arpc::Status IngressConnect(arpc::ServerContext* context, const IngressConnectRequest* request, IngressConnectResponse* response) {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this implementation");
  }

  virtual arpc::Status List(arpc::ServerContext* context, const ListRequest* request, arpc::ServerWriter<ListResponse>* writer) {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this implementation");
  }

  virtual arpc::Status ResolverStart(arpc::ServerContext* context, const ResolverStartRequest* request, ResolverStartResponse* response) {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this implementation");
  }

  virtual arpc::Status ServerStart(arpc::ServerContext* context, const ServerStartRequest* request, ServerStartResponse* response) {
    return arpc::Status(arpc::StatusCode::UNIMPLEMENTED, "Operation not provided by this implementation");
  }
};

class Stub {
 public:
  explicit Stub(const std::shared_ptr<arpc::Channel>& channel)
      : channel_(channel) {}

  arpc::Status ClientConnect(arpc::ClientContext* context, const ClientConnectRequest& request, ClientConnectResponse* response) {
    return channel_->BlockingUnaryCall(arpc::RpcMethod("Switchboard", "ClientConnect"), context, request, response);
  }

  arpc::Status Constrain(arpc::ClientContext* context, const ConstrainRequest& request, ConstrainResponse* response) {
    return channel_->BlockingUnaryCall(arpc::RpcMethod("Switchboard", "Constrain"), context, request, response);
  }

  arpc::Status EgressStart(arpc::ClientContext* context, const EgressStartRequest& request, EgressStartResponse* response) {
    return channel_->BlockingUnaryCall(arpc::RpcMethod("Switchboard", "EgressStart"), context, request, response);
  }

  arpc::Status IngressConnect(arpc::ClientContext* context, const IngressConnectRequest& request, IngressConnectResponse* response) {
    return channel_->BlockingUnaryCall(arpc::RpcMethod("Switchboard", "IngressConnect"), context, request, response);
  }

  std::unique_ptr<arpc::ClientReader<ListResponse>> List(arpc::ClientContext* context, const ListRequest& request) {
    return std::make_unique<arpc::ClientReader<ListResponse>>(channel_.get(), arpc::RpcMethod("Switchboard", "List"), context, request);
  }

  arpc::Status ResolverStart(arpc::ClientContext* context, const ResolverStartRequest& request, ResolverStartResponse* response) {
    return channel_->BlockingUnaryCall(arpc::RpcMethod("Switchboard", "ResolverStart"), context, request, response);
  }

  arpc::Status ServerStart(arpc::ClientContext* context, const ServerStartRequest& request, ServerStartResponse* response) {
    return channel_->BlockingUnaryCall(arpc::RpcMethod("Switchboard", "ServerStart"), context, request, response);
  }

 private:
  const std::shared_ptr<arpc::Channel> channel_;
};

static std::unique_ptr<Stub> NewStub(const std::shared_ptr<arpc::Channel>& channel) {
  return std::make_unique<Stub>(channel);
}

};

}  // namespace switchboard
}  // namespace protocol
}  // namespace flower

#endif
