#ifndef APROTOC_71b2cb1a6bcb60b50ce0ee520756af9b9fe3161f23bf0815b2bac644fe639306
#define APROTOC_71b2cb1a6bcb60b50ce0ee520756af9b9fe3161f23bf0815b2bac644fe639306

#include <cstdint>
#include <map>
#include <memory>
#include <string>
#include <string_view>
#include <vector>

#include <argdata.h>
#include <arpc++/arpc++.h>

namespace flower {
namespace switchboard {

class Configuration final : public arpc::Message {
 public:
  Configuration() : worker_pool_size_(0) {}

  const argdata_t* Build(arpc::ArgdataBuilder* argdata_builder) const override {
    std::vector<const argdata_t*> keys;
    std::vector<const argdata_t*> values;
    if (initial_channel_) {
      keys.push_back(argdata_builder->BuildStr("initial_channel"));
      values.push_back(argdata_builder->BuildFd(initial_channel_));
    }
    if (listening_socket_) {
      keys.push_back(argdata_builder->BuildStr("listening_socket"));
      values.push_back(argdata_builder->BuildFd(listening_socket_));
    }
    if (logger_output_) {
      keys.push_back(argdata_builder->BuildStr("logger_output"));
      values.push_back(argdata_builder->BuildFd(logger_output_));
    }
    if (worker_pool_size_ != 0) {
      keys.push_back(argdata_builder->BuildStr("worker_pool_size"));
      values.push_back(argdata_builder->BuildInt(worker_pool_size_));
    }
    return argdata_builder->BuildMap(std::move(keys), std::move(values));
  }

  void Clear() override {
    *this = Configuration();
  }

  void Parse(const argdata_t& ad, arpc::ArgdataParser* argdata_parser) override {
    argdata_map_iterator_t it;
    argdata_map_iterate(&ad, &it);
    const argdata_t* key;
    const argdata_t* value;
    while (argdata_map_get(&it, &key, &value)) {
      const char* keystr;
      std::size_t keylen;
      if (argdata_get_str(key, &keystr, &keylen) == 0) {
        std::string_view keyss(keystr, keylen);
        if (keyss == "initial_channel") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            initial_channel_ = std::move(fd);
        } else if (keyss == "listening_socket") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            listening_socket_ = std::move(fd);
        } else if (keyss == "logger_output") {
          std::shared_ptr<arpc::FileDescriptor> fd = argdata_parser->ParseFileDescriptor(*value);
          if (fd)
            logger_output_ = std::move(fd);
        } else if (keyss == "worker_pool_size") {
          argdata_get_int(value, &worker_pool_size_);
        }
      }
      argdata_map_next(&it);
    }
  }

  const std::shared_ptr<arpc::FileDescriptor>& initial_channel() const { return initial_channel_; }
  void set_initial_channel(const std::shared_ptr<arpc::FileDescriptor>& value) { initial_channel_ = value; }
  void clear_initial_channel() { initial_channel_.reset(); }

  const std::shared_ptr<arpc::FileDescriptor>& listening_socket() const { return listening_socket_; }
  void set_listening_socket(const std::shared_ptr<arpc::FileDescriptor>& value) { listening_socket_ = value; }
  void clear_listening_socket() { listening_socket_.reset(); }

  const std::shared_ptr<arpc::FileDescriptor>& logger_output() const { return logger_output_; }
  void set_logger_output(const std::shared_ptr<arpc::FileDescriptor>& value) { logger_output_ = value; }
  void clear_logger_output() { logger_output_.reset(); }

  std::uint32_t worker_pool_size() const { return worker_pool_size_; }
  void set_worker_pool_size(std::uint32_t value) { worker_pool_size_ = value; }
  void clear_worker_pool_size() { worker_pool_size_ = 0; }

 private:
  std::shared_ptr<arpc::FileDescriptor> initial_channel_;
  std::shared_ptr<arpc::FileDescriptor> listening_socket_;
  std::shared_ptr<arpc::FileDescriptor> logger_output_;
  std::uint32_t worker_pool_size_;
};

}  // namespace switchboard
}  // namespace flower

#endif
